#include <stdio.h>

#if defined(_HC11)
#include <hc11.h>
#elif defined(_HC12)
#include <hc12.h>
#elif defined(_HC16)
#include <hc16.h>
#else
#error "no target defined"
#endif


/*  Operational Parameters	*/

#define _100MS		25
#define _250MS		61
#define _500MS		125
#define _1SECOND	250
#define _2SECONDS	500
#define _3SECONDS	750
#define _5SECONDS	1250
#define _10SECONDS	2500
#define _25SECONDS	6250
#define _60SECONDS	15000
#define _2MIN		30000

#define TONE_2K		500		/* frequencies in Hertz for tone generation	*/
#define TONE_1K		1000
#define TONE_440	2272
#define TONE_220	4545
#define TONE_180	3000
#define TONE_110	9090
#define TONE_60		16666


/*	Operational Constants	*/

#define TRUE    0xFF
#define FALSE   0x00
#define CR		0x0D


#define opts  	0x90	/* a/d on */
#define admask1	0x30	/* (adctl)continuous conversion, AD0 thru AD3		*/
#define admask2	0x34	/* (adctl)continuous conversion, AD4 thru AD7		*/
#define tmask	0x40	/* (tmsk2)no timer prescale, RTI enabled			*/
#define rtimsk	0x08	/* (pactl) 4.1 msec interrupt with 8 MHz xtal		*/
#define tc1msk	0x40	/*	OC2 in toggle mode								*/

#define clrmsk	0xC0	/*	mask for clearing timer flags					*/
#define ptdmsk	0x02	/*	(ddrd)portd data direction: input, except txd  	*/
#define sc2msk2 0x2C	/*	(sccr2)enable tx & rx and Rx Interrupt			*/


#pragma	interrupt_handler SCIHandler;
#pragma	interrupy_handler OC2Handler;
#pragma	interrupy_handler RTIHandler;


void PulseXToneB(unsigned short x, unsigned short y, unsigned short z);
void EnableToneX(unsigned short x);
void ProcessCommand(void);
void Pcc(void);
void Pcad(unsigned char x, char *s);
void ShowAnalog(void);
void ToggleOutLine(int i);
void ShowMenu(void);

/* System Variables	*/

volatile unsigned char COMMAND, COMMAND_PENDING;
volatile unsigned short TIMER, HALF_PERIOD;

unsigned char CONTINUOUS;



void main(void)
	{
	
	INTR_OFF();			/*	disable interrupts */
	
	OPTION = opts ;		/*	define system option	*/
	ADCTL = admask1 ;	/*	set A/D to scan AD0 thru AD3 continuously	*/        
	
	TMSK2 = tmask ;		/*	enable real-time interrupt (RTI)	*/
	PACTL = rtimsk;		/*	initialize RTI rate	*/
	
	DDRD = ptdmsk ;		/*	initialize PORTD	*/
	
	setbaud(BAUD2400);	/*	baud=2400	*/
	SCCR2=sc2msk2;		/*	enable SCI rcvr. & xmtr. & rx int. */
	/* getchar(); */
	
	COMMAND = 0 ;
	CONTINUOUS = 0 ;
	TIMER = 0 ;
	
	INTR_ON();			/*	enable interrupts */
	
	
	PulseXToneB(2, TONE_440, _100MS); 
	
	
		
	while(1)
		{
				
		ProcessCommand();
		
		if(CONTINUOUS)
			{
			ShowAnalog();
			}		
		}
	}


/* x contains number of beeps desired
   y contains tone frequency
   z contains tone duration (eg. if y=1  => duration=4.1 ms; y=255 => 1 sec)
*/
void PulseXToneB(unsigned short x, unsigned short y, unsigned short z)
	{
	unsigned short i;
	
	for(i=0; i<x; i++)
		{
	
		EnableToneX(y);		/* tone generated via outputCompare	*/
		
		TIMER = z ;
	
		while(TIMER) ;
	
		TMSK1 = 0 ;
		TCTL1 = 0 ;
	
		TIMER = z ;
		while(TIMER) ;
		
		}
		
	}
	 
	
	
void EnableToneX(unsigned short x)
	{
		
	INTR_OFF();
	HALF_PERIOD = x ;
	TCTL1 = tc1msk ;
	TFLG1 = tc1msk ;
	TMSK1 = tc1msk ;
	INTR_ON();
	
	}


void ProcessCommand(void)
	{
		
	if(COMMAND_PENDING)
		{
			
		COMMAND_PENDING = 0;
		CONTINUOUS = 0;
		putchar('\n'); 
		COMMAND |= 0X20;
			
		switch(COMMAND)
			{  
			
			case 'a':
				
				Pcad(PORTA, "	PORTA = ");				
				break ;
				
			case 'c':
			
				Pcc();
				break ;
				
			case 'd':
			
				Pcad(PORTD, "	PORTD = ");
				break ;
				
			case 'r':
			
				CONTINUOUS = TRUE ;	/* set continuous a/d display flag*/
				break ;
			
			case 's':
			
				puts("	>>> BEEP! <<<");
				PulseXToneB(2, TONE_440, _100MS);
				break ; 
					
			case '4':
				
				puts("	PA4 toggled");
				ToggleOutLine(4) ;
				break ;
			
			case '5':
			
			
				puts("	PA5 toggled");			
				ToggleOutLine(5) ;
				break ;
				
			case '6':
			
				puts("	PA6 toggled");
				ToggleOutLine(6);
				break ;			
					
			default:
			
				ShowMenu();
			
			}
		}
	}
	






void Pcad(unsigned char x, char *s)
	{
	
	unsigned char y;
	int i;
	
	for (i = 0; *s; i++, s++){putchar(*s);}

	
	for(i=7; i>=0; i--)
		{
		y = (x>>i) & 0x01 ;
		y += 0x30 ;				
		putchar(y);
		}
		
	putchar('\n');
	
	}	
	
	
void Pcc(void)
	{
	
	PORTA = 0 ;
	puts("	PORTA OUTPUTS CLEARED");
	
	}
	
	
void ShowAnalog(void)
	{
	
		
	ADCTL = admask1 ;
	
	printf("  AND0=%3d",ADR1);
	printf("  AND1=%3d",ADR2);
	printf("  AND2=%3d",ADR3);
	printf("  AND3=%3d",ADR4);
	
	ADCTL = admask2 ;
	
	printf("  AND4=%3d",ADR1);
	printf("  AND5=%3d",ADR2);
	printf("  AND6=%3d",ADR3);
	printf("  AND7=%3d",ADR4);
	
	putchar(CR);
	
	}


	
void ToggleOutLine(int i)
	{
	
	int x;
	x = 0x01 << i;
	PORTA ^= x ;
	
	} 	
 
void ShowMenu(void)
	{
	
	puts("\n	_________________________________________\n");	
	puts(" 	MICROCORE-11 DEMO PROGRAM COMMAND MENU" );
	puts("	_________________________________________");	
		
	puts("	A  => SHOW PORT A STATUS");
	puts("	C  => CLEAR PORT A OUTPUTS");
	puts("	D  => SHOW PORT D STATUS");
	puts("	R  => SHOW REAL-TIME ANALOG VALUES");
	puts("	S  => BEEP SPEAKER (CONNECT TO PA6)");
	puts("	4 TO 6 => TOGGLE THE SELECTED PORTA OUTPUT\n");
	putchar('?');
	
	}



/*  SCI Input Interrupt Handler

	Gets bytes from SCI.  Sets COMMAND_PENDING flag.	*/


void SCIHandler(void)
	{
	
	COMMAND = getchar();
	COMMAND_PENDING = TRUE ;

	}



/*  Output Compare Interrupt Handler	*/

void OC2Handler(void)
	{
	/*puts("test ");*/
	TOC2 += HALF_PERIOD ;
	TFLG1 = tc1msk ; 
	asm("rti");
	}
	
	
	


/*	Real-time Interrupt Routine	*/

void RTIHandler(void)
	{
	
	/*puts("test ");*/
	TFLG2 = 0x40 ;		/*	clear real-time interrupt flag	*/
	if(TIMER) TIMER--;
	asm("rti");
			
	}	





















/* If you need to set up interrupt vectors (e.g. single chip mode system
 * or system without monitor, then you can simply include vectors.c. The
 * "right" way to do things is to set up a project with mulitple files,
 * but to just try out the compiler, this works just as well.
 */
#ifdef _HC16
#include "vectrs16.c"
#else
#include "vectors.c"
#endif
/* note that since vectors.c uses pragma to change the text section name
 * there should not be stuff after this unless you change the name
 * back
 */


